﻿' Script : ot_monitor script
' Purpose: communication with Opentherm (OT) Gateway
' Author : BLB
' Version: 02l 20131027 adapted for OTGW firmware version 4.0a10; PR reply changed from only result to "PR: A=x"; reply PS=1 command striped new due to PS in Reply
' Version: 02k 20130923 adapted for OTGW firmware version 4.0a7; GW command changed from G to M, result G or M
' Version: 02j 20130731 adapted for OTGW firmware version 4.0a6; IT and PR=T command not needed & not supported anymore
' Version: 02i 20130609 corrected opening serial port, port was not open before first message send
' Version: 02h 20130106	added 2nd TempTemperature, the first one with buttons (Thanks Again Rutger!), the second one with the Listbox;
'						changed "devices" section in .ini file to "Temperatures"
'						added default Day and Night temperatures in .ini file;
'						added SerialControlInScript parameter in .ini file to control open/close in script (value TRUE) or in e.g. startup script (value FALSE), so additional setup required!!.
' Version: 02g 20130101	corrected creat OTGW devices when connected to TELNET
' Version: 02f 20121209	corrected debugOTGW values
' Version: 02e 20121209	added create virtal device for temporary temprature control as 7th OTGW Device with Listbox (thanks Rutger);
'						fixed Telnet control (moved communication to Function SendMessage);
'						several clean up actions in the script (e.g. repaces spaces by tabs); debugOTGW settings more consistent;
'						improved execution speed by replacing waitsec(1) with waitMS(200). 
' Version: 02d 20121206 corrected hscontrol to work with temperatures in tenths of decrees and corrected PR=O response;
'						Minor layout improvements PR presentations;
' Version: 02c 20121202	added room & device allocation
' Version: 02b 20121016	changed debug / info setup
' Version: 02a 20111204	added OTGateway devices & create devices if they don't exist in this script
' Version: 01c 20111120	option for communication via Serial port or via Telnet
' Version: 01b 20111026	settings via hspi_OT_monitor.ini
' Version: 01a 20110630	initial release

' Possible Button Commands. Command and Parameter will be separated by "="
'   BU=day (day / COMfort temperature), night (night / ECO temp), Up (0,2 degrees up), Down (o,2 degrees down), Auto (temporary override OFF)

' Possible OT gateway Commands & parameters. Command and Parameter will be seperated by "=" 
'   OT=temperature     Outside temperature - Configures the outside temperature to send to the thermostat. Allowed values are between -40.0 and +127.0, although the Honeywell thermostat won't display values above +40.0.
'                      Examples: OT=21, OT=-3.5 
'   TT=temperature     Temperature, Temporary - Temporarily change the thermostat setpoint. The thermostat program will resume at the next programmed setpoint change. Values between 0.0 and 30.0 are allowed. A value of 0 indicates no remote override is to be applied.
'                      Examples: TT=19.5, TT=0 
'   TC=temperature     Temperature, Constant - Change the thermostat setpoint. The thermostat program will not change this setting. Values between 0.0 and 30.0 are allowed. A value of 0 cancels the remote override.
'                      Examples: TC=16.0, TC=0 
'   TR=temperature     Temperature, Remeha - Change the temperature setpoint on a Remeha Celcia 20, which uses some unusual method for this. A value of 0 cancels the remote override.
'                      Examples: TR=20.2, TR=0 
'   SH=temperature     Setpoint Heating - Set the maximum central heating setpoint. This command is only available with boilers that support this function.
'                      Examples: SH=72.5, SH=+20 
'   SW=temperature     Setpoint Water - Set the domestic hot water setpoint. This command is only available with boilers that support this function.
'                      Examples: SH=60, SH=+40.0 
'   SC=time/day        Set Clock - Change the time and day of the week of the thermostat. The gateway will send the specified time and day of the week in response to the next time and date message from the thermostat. The time must be specified as HH:MM. The day of the week must be specified as a single digit between 1 (Monday) and 7 (Sunday).
'                      Examples: SC=9:00/1, SC=23:59/4 
'   HW=state           Hot Water - Control the domestic hot water enable option. If the boiler has been configured to let the room unit control when to keep a small amount of water preheated, this command can influence that. A state of 0 or 1 will tell the boiler whether or not to keep the water warm. Any other single character causes the gateway to let the thermostat control the boiler. Possible values are 0, 1, or any other single character.
'                      Examples: HW=1, HW=T 
'   AA=Data-ID         Add Alternative - Add the specified Data-ID to the list of alternative commands to send to the boiler instead of a Data-ID that is known to be unsupported by the boiler. Alternative Data-IDs will always be sent to the boiler in a Read-Data request message with the data-value set to zero. The table of alternative Data-IDs is stored in non-volatile memory so it will persist even if the gateway has been powered off. Data-ID values from 1 to 255 are allowed.
'                      Examples: AA=33, AA=117 
'   DA=Data-ID         Delete Alternative - Remove the specified Data-ID from the list of alternative commands. Only one occurrence is deleted. If the Data-ID appears multiple times in the list of alternative commands, this command must be repeated to delete all occurrences. The table of alternative Data-IDs is stored in non-volatile memory so it will persist even if the gateway has been powered off. Data-ID values from 1 to 255 are allowed.
'                      Examples: DA=116, DA=123 
'   LA=function        LED A / LED B / LED C / LED D - These commands can be used to configure the functions of the four LEDs that can optionally be connected to pins RB4-RB7 of the PIC. The following functions are currently available
'   LB=function        Possible functions: R, X, T, B, O, F, H, W, C, E and M
'   LC=function        
'   LD=function        Examples: LC=F, LD=M 
'   PR=item            Print Report - Request the gateway to report some information item. The following items are currently defined (FW 4.0a6 T ceased):
'                      Possible items: A (Build date & time), M (gateway mode), L (LED functions), 
'                                      O (setpoint override value), S (clock speed), T (ignore transitions setting),
'                                      V (reference voltage setting), W (domestic hot water setting)
'                      Examples: PR=L, PR=A
'   VR=level           Voltage Reference - Change the reference voltage used as a threshold for the comparators. This configuration option is stored in non-volatile memory so it will persist even if the gateway has been powered off. The level must be specified as a single digit according to the following table:
'                      Possible levels: 0, 1, 2, 3, 4, 5, 6, 7, 8 and 9.    The normal value is 3.
'                      Examples: VR=3, VR=4 
'   GW=state           GateWay - The opentherm gateway starts up in back-to-back mode. While this is the most useful mode of operation, it also means that the firmware must be able to decode the requests received from the thermostat before it can send them on to the boiler. The same is true for responses from the boiler back to the thermostat. By changing this setting to "0" (monitor mode), the received signal level is passed through to the output driver without any processing. This can be a useful diagnostic tool when there are communication problems immediately after the gateway has been built. See the troubleshooting section for more information.
'                      Examples: GW=0, GW=1 
'   PS=state           Print Summary - The opentherm gateway normally prints every opentherm message it receives, as well as the modified messages it transmits. In some applications it may be more useful to only get a report of the latest values received for the most interesting parameters on demand. Issuing a "PS=1" command will stop the reports for each message and print one status line.
'                      Examples: PS=0, PS=1, PS=auto
'   IT not supported anymore from FW version 4.0a6
'   IT=state           Ignore Transitions - Some thermostats, like the Remeha Celcia20 thermostat, may not produce a perfect Opentherm signal, resulting in many "Error 01" reports by the gateway. When this setting is on (IT=1) the device doesn't check the timing of the non-significant level transitions.

' Additional (combined or special) commands:
'	PS=auto				Will check if outside temp or temptemperature control is changed and execute these commands
'   PR=All             Will ask all PR items
'   OT=outside         Will set the outside temperature to the value of the HS device which is configured in the config file.
'   TR=HSControl       Set the Temporary temperature to the value of the HS device which 

' import required modules for Telnet
Imports System.Net
Imports System.Net.Sockets
Imports System.Text

Const INI_FILE As String = "hssi_BLB_OT_monitor.ini"        ' name ini file
Const swversion = "0.2l"                                    ' the version of this file

Sub Main(ByVal varParameters)
	Dim Reply As String = varParameters
	'Dim RemoteIP, PortNumber, SerialPort, SerialPortConfig As String
	'Dim Telnet As Boolean = False
	
	Dim strHC					As String  = hs.GetINISetting("Startup", "HouseCode", "", INI_FILE)        ' house code
	Dim DevLocation				As String  = hs.GetINISetting("Startup", "DevLocation", "", INI_FILE)
	Dim Telnet					As Boolean = hs.GetINISetting("Startup", "Telnet", "", INI_FILE)
	Dim SCIS					As Boolean = hs.GetINISetting("Startup", "SerialControlInScript", "", INI_FILE)
	Dim RemoteIP				As String  = hs.GetINISetting("Startup", "TCPAddress", "", INI_FILE)
	Dim PortNumber				As String  = hs.GetINISetting("Startup", "TCPPort", "", INI_FILE)
	Dim SerialPort				As Integer = Val(hs.GetINISetting("Startup", "SerialPort", "", INI_FILE))
	Dim SerialPortConfig		As String  = hs.GetINISetting("Startup", "SerialPortConfig", "", INI_FILE)
	Dim Debug					As Integer = hs.GetINISetting("Startup", "Debug", "", INI_FILE)
	Dim strDC					As String  = hs.GetINISetting("OTPSCodes", "FirstDC", "", INI_FILE)
	Dim Celcia20				As Boolean = hs.GetINISetting("Gateway", "Celcia20", "False", INI_FILE)
	Dim LEDS					As String  = hs.GetINISetting("Gateway", "LEDS", "", INI_FILE)
	Dim GWFirstDC				As String  = hs.GetINISetting("Gateway", "FirstDC", "", INI_FILE)
	Dim DevLocsw				As Boolean = hs.GetINISetting("Startup", "DevLocationswitched", "", INI_FILE)
	Dim ReferenceVoltage		As String  = hs.GetINISetting("Gateway", "ReferenceVoltage", "", INI_FILE)
	Dim OutSideTemp				As String  = hs.GetINISetting("Temperatures", "OutsideTemperature", "", INI_FILE)
	Dim TempTemperature1		As String  = hs.GetINISetting("Temperatures", "TemporaryTemperature1", "", INI_FILE)
	Dim TempTemperature2		As String  = hs.GetINISetting("Temperatures", "TemporaryTemperature2", "", INI_FILE)
	Dim TempDayValue			As Integer = Val(hs.GetINISetting("Temperatures", "Day", "", INI_FILE)*10)
	Dim TempNightValue			As Integer = Val(hs.GetINISetting("Temperatures", "Night", "", INI_FILE)*10)
	Dim TempCoolValue			As Integer = Val(hs.GetINISetting("Temperatures", "Cool", "", INI_FILE)*10)	
	Dim strDV, GWValue, GWString, message, otvalue As String
	Dim AllDone 				As Boolean = False
	'test hs.SaveINISetting "my settings","zip code","03110","my_settings.ini        WORKS!!
	'hs.SaveINISetting("ScriptExchange", "TEST", "TEST", INI_FILE)
	

	hs.WriteLog("info","OTWG monitor, Parameters used: " & Reply)
	If Debug > 0 Then 
		hs.WriteLog("debugOTGW","Debug Level: " & CStr(Debug))
		'hs.WriteLog("debugOTGW1","Parameters used: " & Reply)
	End If
	' split Parameter in command and value
	Dim OTGatewayCommand As String = UCase(hs.stringitem(varParameters, 1, "="))
	Dim OTGatewayValue   As String = LCase(hs.stringitem(varParameters, 2, "="))
	Do Until AllDone = True
		If OTGatewayCommand <> "BU" Then            'no button command, so continue with OTGW command
			If not Telnet And SCIS then                      'open serial port if communication via Serial Port and Serial port open/close control in Script
				hs.CloseComPort(SerialPort)         'if serial then close ComPort, maybe comport is still open
				Reply = hs.OpenComPort(SerialPort,SerialPortConfig,0,"","")
				If Reply <> "" Then
					hs.WriteLog("debugOTGW","SendMessage;Error opening COM" & SerialPort & " ,error: " & Reply)
				Else
					If Debug > 0 Then hs.WriteLog("debugOTGW1","SendMessage;opening COM" & Serialport & " port succesful")
				End If
			End If
			' start with a PS=1 command to stop the standard output of OT messages
			Reply = SendMessage("PS=1",Telnet,RemoteIP,PortNumber,SerialPort,SerialPortConfig,Debug)

			
			If OTGatewayValue = "hscontrol" Then               'if true then set temporary temperature from temp device
				OTGatewayValue = TempTemperature2
				otvalue = hs.devicevalue(OTGatewayValue)/10
			Else If  OTGatewayValue = "outside" Then           'if true then set outside temperature to temp outside temperature sensor
				otvalue = replace(hs.devicevalue(OutsideTemp)/10,",", ".")
			Else
				otvalue = OTGatewayValue
			End If
			If Debug > 1 Then hs.WriteLog("debugOTGW2", "OTCommand: "& OTGatewayCommand & " OTValue: " & otvalue)
			
			' configure HS Device if temporary temperature will be set.
			If OTGatewayCommand = "TC" Or OTGatewayCommand = "TR" Or OTGatewayCommand = "TT" Then
				If otvalue = "0" Then                                                                     ' reset value
					hs.setdevicestring(TempTemperature2,"<img src='/images/Sensors/ControlOff.gif'> Off ",True)
				Else
					hs.setdevicestring(TempTemperature2,"<img src='/images/Temperatures/" & CStr(int(otvalue)) & ".gif'> " & otvalue & " °C",True)
					otvalue = replace(otvalue, ",", ".")
				End If
			End if
			
			message = OTGatewayCommand & "=" & otvalue
			If OTGatewayCommand <> "PS" And Debug > 1 Then hs.WriteLog("debugOTGW2", "Send Openthermcommand: " & message)
			' Reply = SendMessage("PS=1",Telnet,RemoteIP,PortNumber,SerialPort,SerialPortConfig,Debug)                 ' set gateway to output only on request

			Try
				If message = "PR=all" Then																' all PR commands
					strDV       = strHC & GWFirstDC
					If hs.DeviceExistsRef(strDV) = -1 Then												' check if OTGW exist. if not, create devices
						Reply = hs.StringItem(SendMessage("PR=A",Telnet,RemoteIP,PortNumber,SerialPort,SerialPortConfig,Debug), 2 , "=")
						CreateOTGWDevices(Reply,SerialPort,GWFirstDC,strHC,DevLocation)
					End If
					' update OTGW Devices
					Dim PRC As String = "T,M,V,O,W,L"						' all relevant PR commands. The right order is T,M,V,O,W,L
					Dim PRCLength As Integer = (PRC.length+1)/2
					Dim PRCounter As Integer
					For PRCounter = 1 To PRCLength
						otvalue  = hs.StringItem(PRC, PRCounter, ",")
						message  = OTGatewayCommand & "=" & otvalue
						Reply = SendMessage(Message,Telnet,RemoteIP,PortNumber,SerialPort,SerialPortConfig,Debug)
						If Debug > 0 Then hs.WriteLog("debugOTGW1", "Reply: " & Reply)
						otvalue = Mid(Reply,5,1)
						Reply = hs.StringItem(Reply, 2 , "=")
						Select Case otvalue
							Case "T"
								GWString = "Normal"
								If Reply = "1" Then GWString = "Celcia20"
								'GWValue  = Int(Reply)
								'GWString = "Normal"
								'If GWValue = 1 Then GWString = "Celcia20" 
							Case "M"
								GWValue = 0
								GWString = "Monitor"
								If asc(Reply) = 71 Then
									GWValue  = 1
									GWString = "Gateway"
								End If
							Case "V"
								GWValue  = Int(Reply)
								GWString = Reply
							Case "O"
								GWValue  = 0
								GWstring = "<img src='/images/uptime/off.gif'> No"
								If left(Reply,1) <> "N"
									GWValue = 1
									GWString = "<img src='/images/uptime/triggeron.gif'> Yes: " & mid(Reply,2,5)
								End If
							Case "W"
								GWValue  = 0
								GWstring = "<img src='/images/uptime/off.gif'> Disabled"
								If left(Reply,1) = "A"
									GWValue = 1
									GWString = "<img src='/images/uptime/on.gif'> Enabled"
								End If
							Case "L"
								GWValue  = 1
								GWString = Reply                        
							Case Else
								hs.WriteLog("debugOTGW","not recognized command: " & otvalue)
						End Select
						hs.SetDeviceValue(strHC & CStr(PRCounter+ Int(GWFirstDC)-1),GWValue)                        
						hs.SetDeviceString(strHC & CStr(PRCounter+ Int(GWFirstDC)-1),GWString)
						If Debug > 1 Then hs.WriteLog("debugOTGW2","Reply on PRC command: " & otvalue & ": " & Reply)
					Next
				Else
					Reply = SendMessage(Message,Telnet,RemoteIP,PortNumber,SerialPort,SerialPortConfig,Debug)
					If Debug > 0 Then hs.WriteLog("debugOTGW1", "Message: " & Message & "Reply: " & Reply)
				End If
				
				If OTGatewayCommand = "PS"
					strDV       = strHC & strDC
					If hs.DeviceExistsRef(strDV) = -1 Then CreateOTDevices(strHC,DevLocation)		' check if First Device exist. if not, create devices 
					Reply = hs.stringitem(Reply, 2, "PS: 1")
					If Debug > 1 Then hs.WriteLog("debugOTGW2", "Reply: " &  Reply)
					Dim k, k1, k2, m As String
					Dim i, i2, j1, j2, j3
					i2 = strDC
					j3 = 2
					For i = 1 To 25
						k = hs.StringItem(Reply, i, ",")
						If i = 1 Or i = 3 Then
							For j1 = 1 To 2
								k1 = Trim(hs.StringItem(k, j1, "/"))
								For j2 = 8 + j3 To 1 + j3 step -1
									k2  = Mid(k1, j2, 1)
									if k2 = "" then k2=0
									m = strHC & CStr(i2)
									If Debug > 2 Then hs.WriteLog("debugOTGW3", "device: #" & m & "# k1: #" & k1 & "# k2: #" & k2 & "# j2: " & j2)
									If i2 < 26 Then
										If k2 = 0 Then
											hs.setdevicestring(m, "<img src='/images/uptime/off.gif'> off")
										Else
											hs.setdevicestring(m, "<img src='/images/uptime/on.gif'> on")
										End If
									ElseIf i2 > 26 And i2 < 35 Then
									If k2 = 0 Then
											hs.setdevicestring(m, "<img src='/images/uptime/off.gif'> transfer disabled")
										Else
											hs.setdevicestring(m, "<img src='/images/uptime/on.gif'> transfer enabled")
										End If
									ElseIf i2 > 34 And i2 < 43 Then
									If k2 = 0 Then
											hs.setdevicestring(m, "<img src='/images/uptime/off.gif'> read-only")
										Else
											hs.setdevicestring(m, "<img src='/images/uptime/on.gif'> read/write")
										End If
									Else
										hs.setdevicestring(m, k2 & " ")
									End If
									hs.setdevicevalue(m, int(k2))
									i2 = i2 + 1
								Next
								j3 = 0
							Next
						ElseIf i = 5 Or i = 14 Or i = 15 Then
							For j1 = 1 To 2
								k1 = Trim(hs.StringItem(k, j1, "/"))
								if k1 = "" then k1=0
								m = strHC & CStr(i2)
								If Debug > 2 Then hs.WriteLog("debugOTGW3", "device: #" & m & "# k1: #" & k1 & "#")
								If i > 5 Then
									hs.setdevicestring(m, "<img src='/images/Temperatures/" & CStr(int(k1)) & ".gif'> " & k1 & " °C")
								Else
									hs.setdevicestring(m, k1 & " ")
								End if
								hs.setdevicevalue(m, k1)   
								i2 = i2 + 1
							Next                
						Else
							k = replace(k, ".", ",")
							if k = "" then k=0
							k = Cstr(Val(k))
							m = strHC & CStr(i2)
							If Debug > 2 Then hs.WriteLog("debugOTGW3", "device (m): #" & m & "# value (k): #" & k & "#")
							If i2 = 26 Or i2 = 46 Or (i2 > 48 And  i2 < 60) Then
								hs.setdevicestring(m, "<img src='/images/Temperatures/" & CStr(int(k)) & ".gif'> " & k & " °C")
							Else
								hs.setdevicestring(m, k & " ")
							End If
							hs.setdevicevalue(m, k)   
							i2 = i2 + 1
						End if
					Next
				End if
			Catch ex As Exception
				hs.WriteLog("OTmonitor error","communication: " & ex.Message)
			End try
			Reply = SendMessage("PS=0",Telnet,RemoteIP,PortNumber,SerialPort,SerialPortConfig,Debug)                 ' set gateway back to default output
			If not Telnet And SCIS Then hs.CloseComPort(SerialPort)
		Else                                          'Buttom commands, Action for Controlbuttons
			If Debug > 2 Then hs.WriteLog("debugOTGW3", "TempTemp2 before: " & hs.devicevalue(TempTemperature2))
			Select Case OTGatewayValue
				Case "night"
					hs.setdevicevalue(TempTemperature2, TempNightValue)
					hs.setdevicestring(TempTemperature1, "Night", True)
				Case "day"
					hs.setdevicevalue(TempTemperature2, TempDayValue)
					hs.setdevicestring(TempTemperature1, "Day", True)
				Case "up"
					If hs.devicevalue(TempTemperature2) <> 0 Then
						hs.setdevicevalue(TempTemperature2, hs.devicevalue(TempTemperature2) + 2)
						hs.setdevicestring(TempTemperature1, (hs.devicevalue(TempTemperature2))/10, True)
					End If
				Case "down"
					If hs.devicevalue(TempTemperature2) <> 0 Then
						hs.setdevicevalue(TempTemperature2, hs.devicevalue(TempTemperature2) - 2)
						hs.setdevicestring(TempTemperature1, (hs.devicevalue(TempTemperature2))/10, True)
					End If
				Case "auto"
					hs.setdevicevalue(TempTemperature2, 0)
					hs.setdevicestring(TempTemperature1, "Auto", True)
				Case "Cool"
					hs.setdevicevalue(TempTemperature2, TempCoolValue)
					hs.setdevicestring(TempTemperature1, "Cool", True)
			
			End Select
			If Debug > 2 Then hs.WriteLog("debugOTGW3", "TempTemp2 after: " & hs.devicevalue(TempTemperature2))
		End IF
		AllDone = True
	Loop
End Sub

Private Function SendMessage(Message,Telnet,RemoteIP,PortNumber,SerialPort,SerialPortConfig,Debug) as String
	Dim Reply as String
	If Debug > 1 Then hs.WriteLog("debugOTGW2","SendMessage;Message: " & Message)
	'hs.CloseComPort(SerialPort)
	If Telnet Then
		If Debug > 2 Then hs.writelog("debugOTGW3","SendMessageTelnet;Message: " & Message)
		Dim Client As New TcpClient()
		Try
			Client.Connect(IPAddress.Parse(RemoteIP), PortNumber)
			Dim NetStream As NetworkStream = Client.GetStream()
			' Send message
			SendString(NetStream, message & vbCrLf)
			' Receive reply
			hsp.WaitMS(200)
			Reply = CleanReply(ReadString(Client),Debug)
			' Disconnect
			Client.Close()
		Catch ex As Exception
			If ex.Message = "De index ligt buiten de matrixgrenzen." Then
				hs.WriteLog("Error","OTGW: " & ex.Message & " You probably need to increase the WaitSec value in the .ini file.")
			Else
				hs.WriteLog("Error","OTGW: " & ex.Message)
			End If
		End Try
	Else
		If Debug > 2 Then hs.writelog("debugOTGW3","SendMessageSerial;Message: " & Message & " SerialPort: " & SerialPort)
		hs.SendToComPort(SerialPort, message & vbCrLf)          ' send command to serial port
		hsp.WaitMS(200)
		Reply = CleanReply(hs.GetComPortData(SerialPort),Debug)             ' read data from serial port
		If Debug > 1 Then hs.WriteLog("debugOTGW2","SendMessage;Response from Serial Port: " & Reply)
	End if
	Return Reply
End Function  

Private Function ReadString(ByVal Client As System.Net.Sockets.TcpClient) As String
	Dim Stream As NetworkStream = Client.GetStream
	Dim Buffersize As Integer = Client.ReceiveBufferSize
	Dim Data(Buffersize -1) As Byte
	If Stream.CanTimeout Then Stream.ReadTimeout = 1000 * 15
	Dim BytesRead As Integer = Stream.Read(Data, 0, Buffersize)
	Return Encoding.ASCII.GetString(Data, 0, BytesRead)
End Function
 
Private Function SendString(ByVal Stream As NetworkStream, ByVal Data As String) As Boolean
	Dim Bytes As [Byte]() = Encoding.ASCII.GetBytes(Data)
	If Stream.CanTimeout Then Stream.WriteTimeout = 1000 * 5
	Stream.Write(Bytes, 0, Bytes.Length)
	Return True
End Function  

Private Function CleanReply(Reply,Debug) as String
	Dim xx = InStr(Reply,"Error")					' check if Reply contains and Error (start from "Error xx" direct after reply on PS=1 command)
	If xx = 1
		If Debug > 1 Then hs.WriteLog("debugOTGW2","Response from Serial Port: " & Reply)
		Reply = Right(Reply,(Reply.Length)-9)		' in that case strip Reply
	Else If xx > 1
		If Debug > 1 Then hs.WriteLog("debugOTGW2","Response from Serial Port: " & Reply)
		Reply = Left(Reply,xx-1)					' in that case strip Reply
	End If
	Return Reply
End Function  

Private Function CreateOTGWDevices(Reply,SerialPort,GWFirstDC,strHC,DevLocation)
	Dim n1(8), C1, dv						'Device Names for Opentherm Gateway (n1)
	Dim DevName00 As String
	n1(1)= "Gateway Type"					'PR=T 
	n1(2)= "Mode"							'PR=M
	n1(3)= "Voltage Reference"				'PR=V
	n1(4)= "Setpoint Override"				'PR=O
	n1(5)= "DHW Enabled"					'PR=W 
	n1(6)= "LEDs"							'PR=L
	n1(7)= "Kamerthermostaat - tijdelijk buttons"
	n1(8)= "Kamerthermostaat - tijdelijk list"
	
	Dim DevLocation2	As String	= hs.GetINISetting("Gateway", "DevLocation2", "", INI_FILE)
	Dim DevLocsw		As Boolean	= hs.GetINISetting("Startup", "DevLocationswitched", "", INI_FILE)
	
	For C1 = 1 To 8
		dv                 = hs.GetDeviceByRef(hs.NewDeviceRef(n1(C1)))
		dv.dc              = CStr(C1+ Int(GWFirstDC)-1)
		dv.hc              = strHC
		dv.can_dim         = False
		if DevLocsw
			dv.location        = DevLocation
			dv.location2       = DevLocation2
		else
			dv.location        = DevLocation2
			dv.location2       = DevLocation
		End If
		dv.dev_type_string = Reply
		dv.misc            = &h10
		Select Case C1
		Case 7
			' Create Controlbuttons: Night, Day, Up, Down, Auto
			hs.devicebuttonadd(strHC & CStr(C1), "BLB_OT_monitor_02.vb(""Main"", ""BU=night"")", "Night")
			hs.devicebuttonadd(strHC & CStr(C1), "BLB_OT_monitor_02.vb(""Main"", ""BU=day"")", "Day")
			hs.devicebuttonadd(strHC & CStr(C1), "BLB_OT_monitor_02.vb(""Main"", ""BU=up"")", "Up")
			hs.devicebuttonadd(strHC & CStr(C1), "BLB_OT_monitor_02.vb(""Main"", ""BU=down"")", "Down")
			hs.devicebuttonadd(strHC & CStr(C1), "BLB_OT_monitor_02.vb(""Main"", ""BU=auto"")", "Auto")
			hs.devicebuttonadd(strHC & CStr(C1), "BLB_OT_monitor_02.vb(""Main"", ""BU=Cool"")", "Cool")
		Case 8
			' Create listbox for Kamerthermostaat - tijdelijk
			hs.DeviceValuesAdd(strHC & CStr(C1), _
			"OFF" & chr(2) & "0" & chr(1) & _
			"06.0" & chr(2) & "060" & chr(1) & _
			"07.0" & chr(2) & "070" & chr(1) & _
			"08.0" & chr(2) & "080" & chr(1) & _
			"09.0" & chr(2) & "090" & chr(1) & _
			"10.0" & chr(2) & "100" & chr(1) & _
			"11.0" & chr(2) & "110" & chr(1) & _
			"12.0" & chr(2) & "120" & chr(1) & _
			"13.0" & chr(2) & "130" & chr(1) & _
			"14.0" & chr(2) & "140" & chr(1) & _
			"15.0" & chr(2) & "150" & chr(1) & _
			"16.0" & chr(2) & "160" & chr(1) & _
			"17.0" & chr(2) & "170" & chr(1) & _
			"18.0" & chr(2) & "180" & chr(1) & _
			"18.5" & chr(2) & "185" & chr(1) & _
			"19.0" & chr(2) & "190" & chr(1) & _
			"19.2" & chr(2) & "192" & chr(1) & _
			"19.4" & chr(2) & "194" & chr(1) & _
			"19.6" & chr(2) & "196" & chr(1) & _
			"19.8" & chr(2) & "198" & chr(1) & _
			"20.0" & chr(2) & "200" & chr(1) & _
			"20.2" & chr(2) & "202" & chr(1) & _
			"20.4" & chr(2) & "204" & chr(1) & _
			"20.6" & chr(2) & "206" & chr(1) & _
			"20.8" & chr(2) & "208" & chr(1) & _
			"21.0" & chr(2) & "210" & chr(1) & _
			"21.2" & chr(2) & "212" & chr(1) & _
			"21.4" & chr(2) & "214" & chr(1) & _
			"21.6" & chr(2) & "216" & chr(1) & _
			"21.8" & chr(2) & "218" & chr(1) & _
			"22.0" & chr(2) & "220" & chr(1) & _
			"22.5" & chr(2) & "225" & chr(1) & _
			"23.0" & chr(2) & "230" & chr(1) & _
			"23.5" & chr(2) & "235" & chr(1) & _
			"24.0" & chr(2) & "240" & chr(1) & _
			"24.5" & chr(2) & "245" & chr(1) & _
			"25.0" & chr(2) & "250" & chr(1) & _
			"Any Value" & chr(2) & "999", true)
		End Select
	Next

End Function  

Private Function CreateOTDevices(strHC,DevLocation)

	Dim DevLocation2 As String  = hs.GetINISetting("OTPSCodes", "DevLocation2", "", INI_FILE)
	Dim Devtype      As String  = hs.GetINISetting("OTPSCodes", "dev_type_string", "", INI_FILE)
	Dim DevLocsw	 As Boolean = hs.GetINISetting("Startup", "DevLocationswitched", "", INI_FILE)
	
	Dim Ref, dv, i, n(58)
	
	n(1) = "000M0 CH enable"
	n(2) = "000M1 HW enable"
	n(3) = "000M2 Cooling enable"
	n(4) = "000M3 OTC active"
	n(5) = "000M4 CH2 enable"
	n(6) = "000M5 reserved" 
	n(7) = "000M6 reserved"
	n(8) = "000M7 reserved"
	n(9) = "000S0 fault indication"
	n(10)= "000S1 CH mode"
	n(11)= "000S2 HW mode"
	n(12)= "000S3 Flame status"
	n(13)= "000S4 Cooling status"
	n(14)= "000S5 CH2 mode"
	n(15)= "000S6 diagnostic indication"
	n(16)= "000S7 reserved"
	n(17)= "001   Control setpoint"
	n(18)= "006T0 HW setpoint"
	n(19)= "006T1 max CH setpoint"
	n(20)= "006T2 reserved"
	n(21)= "006T3 reserved"
	n(22)= "006T4 reserved"
	n(23)= "006T5 reserved"
	n(24)= "006T6 reserved"
	n(25)= "006T7 reserved"
	n(26)= "006W0 HW setpoint"
	n(27)= "006W1 max CH setpoint"
	n(28)= "006W2 reserved"
	n(29)= "006W3 reserved"
	n(30)= "006W4 reserved"
	n(31)= "006W5 reserved"
	n(32)= "006W6 reserved"
	n(33)= "006W7 reserved"
	n(34)= "014   Max rel mod level"
	n(35)= "015H  Boiler capacity (kW)"
	n(36)= "015L  modulation limit (%)"
	n(37)= "016   Room Setpoint"
	n(38)= "017   Rel modulation level"
	n(39)= "018   CH water pressure"
	n(40)= "024   Room temperature"
	n(41)= "025   CH water temp (C)"
	n(42)= "026   HW temperature"
	n(43)= "027   Outside temperature"
	n(44)= "028   Return water temp (C)"
	n(45)= "048H  HW setpoint max (C)"
	n(46)= "048L  HW setpoint min (C)"
	n(47)= "049H  CH  setpoint max (C)"
	n(48)= "049L  CH  setpoint min (C)"
	n(49)= "056   HW setpoint"
	n(50)= "057   Max CH water setpoint"
	n(51)= "116   Burner starts"
	n(52)= "117   CH pump starts"
	n(53)= "118   HW pump/valve starts"
	n(54)= "119   HW burner starts"
	n(55)= "120   Burner operation (h)"
	n(56)= "121   CH pump operation (h)"
	n(57)= "122   HW pump operation (h)"
	n(58)= "123   HW burner operation (h)"
	
	For i = 1 To 58
		Ref = hs.NewDeviceRef(n(i))
		dv = hs.GetDeviceByRef(Ref)
		dv.dc = CStr(9+i)
		dv.hc = strHC
		dv.can_dim = False
		if DevLocsw
			dv.location        = DevLocation
			dv.location2       = DevLocation2
		else
			dv.location        = DevLocation2
			dv.location2       = DevLocation
		End If
		dv.dev_type_string = DevType
		dv.misc = &h10
	Next

End Function                
